/*
 * Created on 14-jul-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.gvsig.geoprocessing.gui.operationpanels;

import java.awt.Component;
import java.io.File;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.event.ListSelectionListener;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.utiles.GenericFileFilter;

public class GeoProcessingMergePanel extends JPanel {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private JLabel titleJLabel = null;
    private JLabel inputJLabel = null;
    private JScrollPane inputJScrollPane = null;
    private JList inputJList = null;
    private JLabel fieldsJLabel = null;
    private JComboBox fieldsJComboBox = null;
    private JLabel outputJLabel = null;
    private JTextField outputJTextField = null;
    private JButton outputJButton = null;
    private FLayers layers = null;
    private FLayer[] layersSelected = null;
    private File outputFile = null;
    /**
     * This is the default constructor
     */
    /*public GeoProcessingMergePanel() {
        super();
        initialize();
    }*/

    /**
     * This constructor initializes the set of layers
     */
    public GeoProcessingMergePanel(FLayers layers) {
        super();
        this.layers = layers;
        initialize();
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        outputJLabel = new JLabel();
        outputJLabel.setBounds(5, 179, 115, 22);
        outputJLabel.setText(PluginServices.getText(this,"Cobertura_de_salida") + ":");
        fieldsJLabel = new JLabel();
        fieldsJLabel.setBounds(5, 148, 160, 22);
        fieldsJLabel.setText(PluginServices.getText(this,"Usar_los_campos_de_la_capa") + ":");
        inputJLabel = new JLabel();
        inputJLabel.setBounds(5, 37, 117, 22);
        inputJLabel.setText(PluginServices.getText(this,"Coberturas de entrada") + ":");
        titleJLabel = new JLabel();
        titleJLabel.setBounds(5, 5, 169, 22);
        titleJLabel.setText(PluginServices.getText(this,"Juntar._Introduccion_de_datos") + ":");
        this.setLayout(null);
        this.setSize(387, 206);
        this.add(titleJLabel, null);
        this.add(inputJLabel, null);
        this.add(getInputJScrollPane(), null);
        this.add(fieldsJLabel, null);
        this.add(getFieldsJComboBox(), null);
        this.add(outputJLabel, null);
        this.add(getOutputJTextField(), null);
        this.add(getOutputJButton(), null);
    }

    private String[] getLayerNames() {
        String[] layerNames = new String[layers.getLayersCount()];
        for (int i=0;i<layers.getLayersCount();i++) {
            layerNames[i] = layers.getLayer(i).getName();
        }
        return layerNames;
    }

    private String[] getLayersSelectedNames() {
        String[] layersSelectedNames = new String[layersSelected.length];
        for (int i=0;i<layersSelected.length;i++) {
            layersSelectedNames[i] = layersSelected[i].getName();
        }
        return layersSelectedNames;
    }

    /**
     * This method initializes inputJScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */    
    private JScrollPane getInputJScrollPane() {
    	if (inputJScrollPane == null) {
    		inputJScrollPane = new JScrollPane();
    		inputJScrollPane.setBounds(127, 37, 254, 102);
    		inputJScrollPane.setViewportView(getInputJList());
    	}
    	return inputJScrollPane;
    }

    /**
     * This method initializes inputJList	
     * 	
     * @return javax.swing.JList	
     */    
    private JList getInputJList() {
        if (inputJList == null) {
            inputJList = new JList(getLayerNames());
            LayerSelectorListener layerSelectorListener = new LayerSelectorListener(layers, getLayerNames());
            inputJList.addListSelectionListener(layerSelectorListener);
            /*String[] selectedLayerNames = (String[])jComboBox.getSelectedObjects();
            inputs = new FLayer[selectedLayerNames.length];
            for (int i=0;i<selectedLayerNames.length;i++) {
                String selectedLayerNamei = selectedLayerNames[i];
                inputs[i] = layers.getLayer(selectedLayerNamei);
            }*/
        }
        return inputJList;
    }

    private class LayerSelectorListener implements ListSelectionListener {
        private FLayers layers;
        private String[] layerNames;
        public LayerSelectorListener(FLayers layers, String[] layerNames) {
            this.layers = layers;
            this.layerNames = layerNames;
        }
        public void valueChanged(javax.swing.event.ListSelectionEvent e) {    
            int[] indices = inputJList.getSelectedIndices();
            String[] layersSelectedNames = new String[indices.length];
            for (int i=0;i<indices.length;i++) {
                int index = indices[i];
                for (int j=0;j<layerNames.length;j++) {
                    if (j==index) {
                        layersSelectedNames[i] = layerNames[j];
                        break;
                    }
                }
            }
            layersSelected = new FLayer[layersSelectedNames.length];
            for (int i=0;i<layersSelectedNames.length;i++) {
                String layerString = layersSelectedNames[i];
                for (int j=0;j<layers.getLayersCount();j++) {
                    FLayer layer = layers.getLayer(j);
                    if (layerString.equals(layer.getName())) {
                        layersSelected[i] = layer;
                    }
                }
            }
            DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getLayersSelectedNames());
            fieldsJComboBox.setModel(defaultModel);
        }
    }

    /**
     * This method initializes fieldsJComboBox	
     * 	
     * @return javax.swing.JComboBox	
     */    
    private JComboBox getFieldsJComboBox() {
    	if (fieldsJComboBox == null) {
    		fieldsJComboBox = new JComboBox();
            //DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getLayersSelectedNames());
            //fieldsJComboBox.setModel(defaultModel);
    		fieldsJComboBox.setBounds(173, 148, 210, 22);
    	}
    	return fieldsJComboBox;
    }

    /**
     * This method initializes outputJTextField	
     * 	
     * @return javax.swing.JTextField	
     */    
    private JTextField getOutputJTextField() {
    	if (outputJTextField == null) {
    		outputJTextField = new JTextField();
    		outputJTextField.setBounds(128, 179, 183, 22);
    	}
    	return outputJTextField;
    }

    /**
     * This method initializes outputJButton	
     * 	
     * @return javax.swing.JButton	
     */    
    private JButton getOutputJButton() {
    	if (outputJButton == null) {
    		outputJButton = new JButton();
    		outputJButton.setText(PluginServices.getText(this,"Abrir"));
            outputJButton.addActionListener(new java.awt.event.ActionListener() { 
                public void actionPerformed(java.awt.event.ActionEvent e) {    
                    System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
                    JFileChooser jfc = new JFileChooser();
                    jfc.addChoosableFileFilter(new GenericFileFilter("shp", "Ficheros SHP"));
                    if (jfc.showSaveDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
                        File file = jfc.getSelectedFile();
                        if (!(file.getPath().endsWith(".shp") || file.getPath().endsWith(".SHP"))){
                            file = new File(file.getPath()+".shp");
                        }
                        outputFile = file;
                    }
                    if (jfc.getSelectedFile()!=null) getOutputJTextField().setText(jfc.getSelectedFile().getAbsolutePath());
                }
            });
            outputJButton.setBounds(320, 179, 63, 22);
    	}
    	return outputJButton;
    }
    /**
     * @return Returns the outputFile.
     */
    public File getOutputFile() {
        return outputFile;
    }

    /**
     * @return Returns the layerSelectedNames.
     */
    public Object[] getLayerSelectedNames() {
        //System.out.println("getInputJList().getSelectedValues().getClass().getName() = " + getInputJList().getSelectedValues().getClass().getName());
        return getInputJList().getSelectedValues();
    }
    
    /**
     * This method returns the name of the layer selected at fieldsJComboBox. The
     * fields of this layer will be the fields of the output layer.
     * @return String
     */
    public String getFieldsFromLayerName() {
        return ((String)getFieldsJComboBox().getSelectedItem());
    }

}  //  @jve:decl-index=0:visual-constraint="10,10"
