/*
 * Created on 06-jul-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.gvsig.geoprocessing.operations;

import java.io.File;
import java.util.BitSet;
import java.util.Vector;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;

public class GeoProcessingTableOperations {
    
    /**
     * Creates the .dbf result file for the buffer operation. This .dbf file
     * corresponds to the .shp that have been created with the
     * saveBufferResultsToShapeFile() method.
     * @param file, output file.
     * @param lyr, input theme table.
     * @param dissolveBuffer, boolean that informs about if the dissolve option is
     * active.
     * @param selection, boolean that informs about if we are working with a selection
     * only.
     * @throws DriverException
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException
     * @throws IncompatibleTypesException
     * @throws DriverLoadException
     * @throws NoSuchTableException
     */
    public static void saveBufferResultsToDbaseFile(File file, AlphanumericData lyr, boolean dissolveBuffer, boolean isDbfFile) 
            throws DriverException, 
            com.hardcode.gdbms.engine.data.driver.DriverException, 
            IncompatibleTypesException, DriverLoadException, NoSuchTableException {
        SelectableDataSource sds = lyr.getRecordset();
        // Tener en cuenta que podemos estar trabajando solo con una seleccin
        FBitSet fBitSet = ((Selectable)lyr).getSelection();
        if (fBitSet.cardinality()==0) fBitSet.set(0, (int)lyr.getRecordset().getRowCount());
        if (dissolveBuffer) saveOnlyCommonFields(sds, fBitSet, file);
        else saveAllFieldsAndRows(sds, fBitSet, file, isDbfFile);
    }
    
    /**
     * This method is executed when we are making a buffer with the dissolve option.
     * Result table will be the same that the input table.
     * @param sds, table of the input theme.
     * @param fBitSet, items selected.
     * @param file, output file.
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException 
     * @throws IncompatibleTypesException 
     * @throws NoSuchTableException 
     * @throws DriverLoadException 
     */
    // TODO: Provisional. Hasta que Fernando le meta PKs al resto de ficheros.
    private static void saveAllFieldsAndRows(SelectableDataSource sds, FBitSet fBitSet, File file, boolean isDbfFile) 
            throws com.hardcode.gdbms.engine.data.driver.DriverException, 
            IncompatibleTypesException, DriverLoadException, NoSuchTableException {
        // Esto cambia si pasamos del dissolve
        String[] fieldNames = new String[sds.getFieldCount()];
        int[] fieldTypes = new int[sds.getFieldCount()];
        for (int i = 0; i < sds.getFieldCount(); i++) {
            fieldNames[i] = sds.getFieldName(i);
            fieldTypes[i] = sds.getFieldType(i);
        }
        //
        DataSourceFactory dsf = sds.getDataSourceFactory();
        dsf.createFileDataSource("gdbms dbf driver", "new_table", file.getAbsolutePath().replaceAll(".shp", ".dbf"), fieldNames, fieldTypes);
        DataSource ds = dsf.createRandomDataSource("new_table");
        ds.start();
        DataWare dw = ds.getDataWare(DataSourceFactory.MANUAL_OPENING);
        dw.beginTrans();
        ds.stop();
        sds.stop();
        // Los DS que vienen de un DBF vienen con PK y los que vienen de un DXF todava no. Mientras esto
        // est as, hay que distinguir entre estos orgenes. Ahora solo se mete PK en los DBFs.
        int fieldCount = 0;
        if (isDbfFile) fieldCount = sds.getFieldCount();
        else fieldCount = sds.getFieldCount()+1;
        Value[][] values = new Value[(int)fBitSet.cardinality()][fieldCount];
        int k=0;
        for (int i=0;i<sds.getRowCount();i++) {
            if (fBitSet.get(i)) {
                for (int j=0;j<fieldCount-1;j++) {
                    values[k][j] = sds.getFieldValue(i, j);
                }
                k++;
            }
        }
        //for (int i=0;i<sds.getRowCount();i++) {
        for (int i=0;i<fBitSet.cardinality();i++) {
            values[i][fieldCount-1] = ValueFactory.createValue(i);
        }
        //
        // Aqu habr que hacer un for o algo as ...
        //for (int i=0;i<dw.getRowCount();i++) {
        for (int i=0;i<fBitSet.cardinality();i++) {
            dw.insertFilledRow(values[i]);
        }
        //
        dw.commitTrans();
    }
    
    /**
     * This method is executed when we are making a buffer without the dissolve option.
     * Result table will have one row corresponding to the only one buffer polygon and
     * only the fields that have common values.
     * @param sds, table of the input theme.
     * @param fBitSet, items selected.
     * @param file, output file.
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException 
     * @throws IncompatibleTypesException 
     * @throws NoSuchTableException 
     * @throws DriverLoadException 
     */
    private static void saveOnlyCommonFields(SelectableDataSource sds, FBitSet fBitSet, File file) 
            throws com.hardcode.gdbms.engine.data.driver.DriverException, 
            IncompatibleTypesException, DriverLoadException, NoSuchTableException {
        // Esto cambia si pasamos del dissolve
        Vector fieldNames = new Vector();
        Vector fieldTypes = new Vector();
        BitSet bitset = new BitSet(sds.getFieldCount());
        for (int i = 0; i < sds.getFieldCount(); i++) {
            //Value[] values = new Value[(int)sds.getRowCount()];
            Value[] values = new Value[(int)fBitSet.cardinality()];
            boolean hasTheSameRowValues = true;
            //for (int j=0;j<sds.getRowCount();j++) {
            //for (int j=0;j<fBitSet.cardinality();j++) {
            int k=0;
            int l=0;
            for (int j=0;j<sds.getRowCount();j++) {
                if (fBitSet.get(j)) {
                    Value value = sds.getFieldValue(j, i);
                    values[l] = value;
                    if (j>0) {
                        if (!((BooleanValue)value.equals(values[j-k])).getValue()) {
                            hasTheSameRowValues = false;
                            break;
                        }
                    }
                    l++;
                }
                k++;
            }
            if (hasTheSameRowValues) {
                fieldNames.add(sds.getFieldName(i));
                fieldTypes.add(new Integer(sds.getFieldType(i)));
                bitset.set(i);
            }
        }
        //
        String[] fieldNamesArray = new String[fieldNames.size()];
        int[] fieldTypesArray = new int[fieldNames.size()];
        for (int i=0;i<fieldNames.size();i++) {
            fieldNamesArray[i] = ((String)fieldNames.get(i));
            fieldTypesArray[i] = (((Integer)fieldTypes.get(i)).intValue());
        }
        DataSourceFactory dsf = sds.getDataSourceFactory();
        dsf.createFileDataSource("gdbms dbf driver", "new_table", file.getAbsolutePath().replaceAll(".shp", ".dbf"), fieldNamesArray, fieldTypesArray);
        DataSource ds = dsf.createRandomDataSource("new_table");
        ds.start();
        DataWare dw = ds.getDataWare(DataSourceFactory.MANUAL_OPENING);
        dw.beginTrans();
        ds.stop();
        sds.stop();
        // Esto tb cambia
        Vector values = new Vector();
        for (int i=0;i<sds.getFieldCount();i++) {
            if (bitset.get(i)) {
                values.add(sds.getFieldValue(0, i));
            }
        }
        values.add(ValueFactory.createValue(0));
        Value[] valuesArray = new Value[values.size()];//+1];
        for (int i=0;i<values.size();i++) {
            valuesArray[i] = ((Value)values.get(i));
        }
        //valuesArray[values.size()] = ValueFactory.createValue(0L);
        //
        // Aqu habr que hacer un for o algo as ...
        dw.insertFilledRow(valuesArray);
        //
        dw.commitTrans();
    }
    
    /**
     * Creates the .dbf result file for the merge operation. This .dbf file
     * corresponds to the .shp that have been created with the
     * saveMergeResultsToShapeFile() method.
     * @param file, output file.
     * @param lyrs, the layers to merge.
     * @param lyr, the fields of the resulting .dbf will be the fields of lyr.
     * @throws DriverException
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException
     * @throws IncompatibleTypesException
     * @throws DriverLoadException
     * @throws NoSuchTableException
     */
    // TODO: Provisional. Hasta que Fernando le meta PKs al resto de ficheros.
    public static void saveMergeResultsToDbaseFile(File file, AlphanumericData[] lyrs, AlphanumericData lyr, boolean isDbfFile) 
            throws DriverException, 
            com.hardcode.gdbms.engine.data.driver.DriverException, 
            IncompatibleTypesException, DriverLoadException, NoSuchTableException {
        SelectableDataSource sds = lyr.getRecordset();
        // Los DS que vienen de un DBF vienen con PK y los que vienen de un DXF todava no. Mientras esto
        // est as, hay que distinguir entre estos orgenes. Ahora solo se mete PK en los DBFs.
        int fieldCount = 0;
        if (isDbfFile) fieldCount = sds.getFieldCount();
        else fieldCount = sds.getFieldCount()+1;
        String[] fieldNames = new String[fieldCount];
        int[] fieldTypes = new int[fieldCount];
        for (int i=0;i<fieldCount;i++) {
            fieldNames[i] = sds.getFieldName(i);
            fieldTypes[i] = sds.getFieldType(i);
        }
        DataSourceFactory dsf = sds.getDataSourceFactory();
        dsf.createFileDataSource("gdbms dbf driver", "new_table", file.getAbsolutePath().replaceAll(".shp", ".dbf"), fieldNames, fieldTypes);
        DataSource ds = dsf.createRandomDataSource("new_table");
        ds.start();
        DataWare dw = ds.getDataWare(DataSourceFactory.MANUAL_OPENING);
        dw.beginTrans();
        ds.stop();
        sds.stop();
        // Tener en cuenta que podemos estar trabajando con selecciones y no con capas
        // enteras.
        FBitSet[] fBitSets = new FBitSet[lyrs.length];//((Selectable)lyr).getSelection();
        for (int i=0;i<lyrs.length;i++) {
            fBitSets[i] = lyrs[i].getRecordset().getSelection();
            if (fBitSets[i].cardinality()==0) {
                fBitSets[i].set(0, (int)lyrs[i].getRecordset().getRowCount());
            }
        }
        int rowNumber = 0;
        for (int i=0;i<lyrs.length;i++) {
            rowNumber = rowNumber + (int)lyrs[i].getRecordset().getSelection().cardinality();
        }
        Value[][] values = new Value[rowNumber][fieldCount];
        // Voy rellenando la tabla por columnas
        int n=0;
        int m=0;
        boolean hasField = false;
        for (int i=0;i<lyrs.length;i++) {
            for (int j=0;j<fieldCount-1;j++) {
                hasField = false;
                for (int k=0;k<lyrs[i].getRecordset().getFieldCount()-1;k++) {
                    m=n;
                    if (fieldNames[j].equals(lyrs[i].getRecordset().getFieldName(k))) {
                        hasField = true;
                        for (int l=0;l<(int)lyrs[i].getRecordset().getRowCount();l++) {
                            if (fBitSets[i].get(l)) {
                                values[m][j] = lyrs[i].getRecordset().getFieldValue(l, lyrs[i].getRecordset().getFieldIndexByName(fieldNames[j])); // fieldNames[k] o fieldNames[j]?
                                m++;
                            }
                        }
                    }
                }
                if (!hasField) {
                    // rellena con nulls ese cacho de columna
                    m=n;
                    for (int l=0;l<(int)lyrs[i].getRecordset().getRowCount();l++) {
                        if (fBitSets[i].get(l)) {
                            values[m][j] = ValueFactory.createNullValue();
                            m++;
                        }
                    }
                }
            }
            //n=(int)lyrs[i].getRecordset().getRowCount();
            n=(int)lyrs[i].getRecordset().getSelection().cardinality();
        }
        for (int i=0;i<rowNumber;i++) {
            values[i][fieldCount-1] = ValueFactory.createValue(i);
        }
        // Chapando todos los SDS antes de rellenar ...
        /*for (int i=0;i<lyrs.length;i++) {
            lyrs[i].getRecordset().stop();
        }*/
        for (int i=0;i<rowNumber;i++) {
            dw.insertFilledRow(values[i]);
        }
        dw.commitTrans();
    }
    
    /**
     * Creates the .dbf result file for the dissolve operation. This .dbf file
     * corresponds to the .shp that have been created with the
     * saveDissolveResultsToShapeFile() method.
     * @param file, output file.
     * @param lyr, input theme table.
     * @param dissolveAttributeName, the name of the attribute for doing the dissolve
     * operation.
     * @throws DriverException
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException
     * @throws IncompatibleTypesException
     * @throws DriverLoadException
     * @throws NoSuchTableException
     */
    public static void saveDissolveResultsToDbaseFile(File file, AlphanumericData lyr, String dissolveAttributeName) 
            throws DriverException, 
            com.hardcode.gdbms.engine.data.driver.DriverException, 
            IncompatibleTypesException, DriverLoadException, NoSuchTableException {
        SelectableDataSource sds = lyr.getRecordset();
        // Tener en cuenta que podemos estar trabajando solo con una seleccin
        FBitSet fBitSet = ((Selectable)lyr).getSelection();
        String[] fieldNames = new String[]{dissolveAttributeName};
        int[] fieldTypes = new int[]{sds.getFieldType(sds.getFieldIndexByName(dissolveAttributeName))};
        Vector values = new Vector();
        for (int i=0;i<sds.getRowCount();i++) {
            if (fBitSet.get(i)) {
                values.add(sds.getFieldValue(i, sds.getFieldIndexByName(dissolveAttributeName)));
            }
        }
        for (int i=0;i<values.size();i++) {
            for (int j=0;j<values.size();j++) {
                if (j>i) {
                    if (((BooleanValue)((Value)values.get(i)).equals((Value)values.get(j))).getValue()) {
                        values.remove(j);
                        j--;
                    }
                }
            }
        }
        Value[][] valuesTable = new Value[values.size()][2];
        DataSourceFactory dsf = sds.getDataSourceFactory();
        dsf.createFileDataSource("gdbms dbf driver", "new_table", file.getAbsolutePath().replaceAll(".shp", ".dbf"), fieldNames, fieldTypes);
        DataSource ds = dsf.createRandomDataSource("new_table");
        ds.start();
        DataWare dw = ds.getDataWare(DataSourceFactory.MANUAL_OPENING);
        dw.beginTrans();
        ds.stop();
        sds.stop();
        for (int i=0;i<valuesTable.length;i++) {
            valuesTable[i][0] = ((Value)values.get(i));
            valuesTable[i][1] = ValueFactory.createValue(i);
        }
        for (int i=0;i<valuesTable.length;i++) {
            dw.insertFilledRow(valuesTable[i]);
        }
        dw.commitTrans();
    }
}
